//
// bidwatcher
// copyright (c) 1999, 2000
// Trent McNair (trent@rmci.net)
// Tom McNair  (tmcnair@cyberhighway.net)
// Wayne Schlitt (wayne@midwestcs.com)
// 
// use of this code is restricted to the terms
// of the GPL, which should have been included in this
// distribution. If not, see www.gnu.org/copyleft/gpl.html.
// Here is the short version:
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
#ifndef AUCTION_H
#define AUCTION_H
#include <qpopupmenu.h>
#include <qtooltip.h>
#include <qpainter.h>
#include <qwidget.h>
#include <qlineedit.h>
#include <qbitmap.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qdialog.h>
#include <qdatetime.h>
#include <qpixmap.h>
#include <qtimer.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include <qspinbox.h>
#include <qmultilinedit.h>
#include <qpalette.h>
#include <qlistview.h>



// This is the amount of time before Bidwatcher will give up on
// fetching a web page.  It is fairly common for something to "hang",
// and a the quickest way to get the page loaded is to give up and
// try again.
//
// If this value is too small, then bidwatcher may give up on a
// request that is actually close to finishing.  By giving up too
// early, you end up having to do two requests, which takes more time.
// It is probably better for this to be too large than too small.
//
// If this value is too large, then precious time will can be wasted
// when a snipe is supposed to be executed.
//
// Actually, bidwatcher will detect hung connections quicker than the
// TIMEOUT value.  This is really more of the "maximum" timeout.
  
const int TIMEOUT = 4000;	// in milliseconds
const QColor bg2(210,210,210);
const QColor bg3(240,240,240);
const int NORMAL_LOAD_TIME = 2000;	// good for ISDN, ADSL, cable, T1 etc.

const int INITIAL_LOAD_TIME = (
    NORMAL_LOAD_TIME		   // normal load time
    + (1000* 20*1024)/(56000/10)   // ms to download a 20k page on 58k modem
    );

extern double avg_load_time;


// other constants
const int HUGEBUFF = (256*1024);
const int MAXAUCS = 100;
const int BIGDELAY = (2*60*60*1000);	// in seconds
const int AUTODEL = (-24*60*60);	// in seconds

// These are structures that hold the IP address of the eBay's web site.
//
// This is a horrible blotch that should be cleaned up.  First, the IP
// address is stored as bunch of characters, and copied into an interger.
// This is not portable, but fortunately it will work on most platforms.
// Secondly, the IP address *should* be looked up via gethostbyname() so
// if eBay changes them, we won't get messed up.  Thirdly, the host names
// following the IP addresses are never used.
const char theAddress[] = { 216, 32, 120, 136,
			    'c','g','i','.','e','b','a','y',
			    '.','c','o','m','\0' };

const char theAddress3[] = { 216, 32, 120, 139,
			     'c','g','i','3','.', 'e','b','a',
			     'y','.','c','o','m','\0' };

const char theAddress6[] = { 216, 32, 120, 159,
                             'c','g','i','6','.', 'e','b','a',
                             'y','.','c','o','m','\0' };
// current version of Bidwatcher
const char versionInfo[]= "2.02";


// number of items in a static array
#define array_elem(a)	(sizeof(a)/sizeof(a[0]))


//
// Stucture to store auction information
//
struct BidMakeup_type {
  char ItemNumber[30];     /*  Item Number (User Entered)    */
  char Description[129];   /*  Description Of Item For sale  */
  char currency[10];
  char BuyitNowPrice[15];
  char CurrentBid[15];     /*  1: Current Bid                */
  char FirstBid[15];       /*  2: First (or starting) Bid    */
  char Quantity[15];       /*  3: Quantity of Items For Sale */
  char BidCount[15];       /*  4: Total Number of Bids       */
  char TimeLeft[51];       /*  5: Time Left in Auction       */
  char Location[101];      /*  6: Location of Item           */
  char Started[51];        /*  7: Time when Auction Started  */
  char Ends[51];           /*  8: Time/Data Auction Ends     */
  char Seller[76];         /*  9: User Selling the Item      */
  char HighBidder[76];     /* 10: Current High Bidder        */
  char BidInc[15];         /* 11: Minimum Bid Increment      */
  char SellerRate[15];
  char BidderRate[15];
  char snipeAmount[15];
  char snipeQty[15];
  char snipeKey[100];
  long int EndsValue;      // this is the time that the auction ends in seconds.
  char stat;
  char reserveMet;         /* y = met, n = not met, x = no reserve */ 
  char BuyitNow;
  bool isSnipe;
  bool auc_type_save;
  int endedNow;
};
///////////////////////////////////////////////////////////////////////////////
//////////////////--------MyListBoxItem Class--------//////////////////////////
///////////////////////////////////////////////////////////////////////////////
class MyListBoxItem : public QListBoxItem
{
public:
   MyListBoxItem( const char *s, const QPixmap p,  QColor textC )
      : QListBoxItem(), pm(p), textColor(textC)
      { setText( s ); }

protected:
   virtual void paint( QPainter * );
   virtual int height( const QListBox * ) const;
   virtual int width( const QListBox * ) const;
   virtual const QPixmap *pixmap() { return &pm; }
   
private:
   QPixmap pm;
   QColor   textColor;
   QPalette        grayPalette;
};
///////////////////////////////////////////////////////////////////////////////
///////////////-----------NewListBox Class-----////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
class NewListBox : public QListBox
{
   Q_OBJECT
public:
   NewListBox( QWidget * parent=0, const char * name=0, WFlags f=0 ); 
signals:
   void RightClicked();
   void MiddleClicked();
protected:
   void mousePressEvent( QMouseEvent * );
};
///////////////////////////////////////////////////////////////////////////////
/////////////---------BidWindow Class----------////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
class BidWindow : public QDialog
{
   Q_OBJECT
public:
   BidWindow(QWidget *, const char *, bool , WFlags);
   void fillBlanks(BidMakeup_type &, int idx, char *, char *, int, char *);
   struct BidMakeup_type theAuction;
signals:
   void setsnipe(BidMakeup_type &, int);
public slots:
   void CmCancel();
   void CmBidNow();
   void CmSnipe();
   void timerUp();
protected:
private:
   QLabel * itemLabel;
   QLabel * descriptionLabel;
   QLabel * sideBarLabel;
   QLabel * priceLabel;
   QLabel * categoryLabel;
   QLabel * infoLabel;
   QLabel * bidLabel; 			// dbs
   QLabel * prevSnipeLabel;
   QLabel * fontLabel;
   QLabel * timeLabel;
   QTimer * DETAIL_TIMER;
   QLineEdit * maxBid;
   QLineEdit * maxQuant;		// dbs
   int theAuctionIdx;
   char userID[76];
   char userPASS[76];
   bool doSnipe;
   QPushButton * cancelBidButton;
   QPushButton * bidButton;
   QPushButton * snipeButton;
   QPalette        grayPalette;
	QPalette        blackPalette;
   QPalette        redPalette;
   int ebayOffset;
}; 
///////////////////////////////////////////////////////////////////////////////
//////////////---------GetNumber Class--------------------------//////////////
///////////////////////////////////////////////////////////////////////////////
class QGetNumber : public QDialog
{
   Q_OBJECT
public:
  	QGetNumber(QWidget * parent=0 , const char * name=0 , bool modal=FALSE, WFlags f=0 );
	~QGetNumber();
	
public slots:
   void CmOK();
   void CmCancel();

signals:
   void submit_number(char *);

private slots:

protected:
   void closeEvent( QCloseEvent * );
   
private:
   QLineEdit * inputEdit;
   QLabel * inputLabel;
   QPushButton * OKButton;
   QPushButton * CancButton;
};
///////////////////////////////////////////////////////////////////////////////
//////////////---------ConfigMenu Class--------------------------//////////////
///////////////////////////////////////////////////////////////////////////////
class ConfigMenu : public QDialog
{
   Q_OBJECT
public:
  	ConfigMenu(QWidget * parent=0 , const char * name=0 , bool modal=FALSE, WFlags f=0 );
	~ConfigMenu();
	void LoadValues(char *, char *, char *, bool , bool ,bool, bool, bool, int, char *, bool, bool, bool, int);
public slots:
   void CmKeep();
   void CmScrap();
signals:
   void its_done(char * , char * , char *, bool , bool , bool , bool, bool, int , char *, bool, bool, bool, int);
private slots:
protected:   
    void closeEvent( QCloseEvent * );
signals:
private:
   QLineEdit * userEdit;
   QLineEdit * passEdit;
   QLineEdit * browserEdit;
   QLineEdit * setebaypEdit;
   QSpinBox *  timeSpinner;
   QSpinBox * fontSpinner;
   QLabel * timeLabel;
   QLabel * fontLabel;
   QLabel * browserLabel;
   QLabel * userLabel;
   QLabel * passLabel;
   QLabel * setebaypLabel;
   QCheckBox * listCheck;
   QCheckBox * bidCheck;
   QCheckBox * alarmCheck;
   QCheckBox * startCheck;
   QCheckBox * deleteCheck;
   QCheckBox * earlyCheck;
   QCheckBox * boldCheck;
   QCheckBox * fixedCheck;
   QPushButton * scrapButton;
   QPushButton * keepButton;
};
///////////////////////////////////////////////////////////////////////////////
/////////////---------DetailsWindow Class----------////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
class DetailsWindow : public QDialog
{
   Q_OBJECT
public:
   DetailsWindow(QWidget *, const char *, bool , WFlags);
   void fillBlanks(struct BidMakeup_type,char*,char*, int);
signals:
public slots:
   void updateStats();
   void CmClose();
   void timerUp();
protected:
private:
   //QLabel * auctionLabel;
   //QLabel * snipeLabel;
   QLabel * itemLabel;
   QLabel * descriptionLabel;
   QLabel * sideBarLabel;
   QLabel * priceLabel;
   QLabel * categoryLabel;
   QLabel * infoLabel;
   QLabel * bidLabel; 		// DBS 
   QTimer * DETAIL_TIMER;
   struct BidMakeup_type theAuction;
   char   tempID[76];
   char   tempPASS[76];
   QPushButton * closeButton;
   QPalette        grayPalette;
	QPalette        blackPalette;
   QPalette        redPalette;
   int ebayOffset;
}; 
///////////////////////////////////////////////////////////////////////////////
/////////////---------QViewFile Class----------////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
class  QFileView : public QDialog
{
   Q_OBJECT
public:
   QFileView(QWidget *, const char *, bool , WFlags);
   void fillBlanks(char * fileToView, bool isItThisMonth);
signals:
public slots:
   void CmClose();
   void CmChangeMonth();
protected:
private:
   QLabel * topLabel;
   QPushButton * closeButton;
   QPushButton * monthButton;
   QMultiLineEdit * viewArea;
   bool thisMonth; // TRUE = this month, FALSE = last month
}; 
///////////////////////////////////////////////////////////////////////////////
/////////////---------QEndedView Class----------////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
class  QEndedView : public QDialog
{
   Q_OBJECT
public:
   QEndedView(QWidget *, const char *, bool , WFlags);
   void fillBlanks(char * fileToView, bool isItThisMonth);
signals:
public slots:
   void CmClose();
   void CmChangeMonth();
protected:
private:
   QLabel * topLabel;
   QPushButton * closeButton;
   QPushButton * monthButton;
   QMultiLineEdit * viewArea;
   bool thisMonth; // TRUE = this month, FALSE = last month
}; 
//////////////////////////////////////////////////////////////////////////////
//////////------------BidWatcher Class--------------////////////////////////
//////////////////////////////////////////////////////////////////////////////
typedef char string30[30];

class BidWatcher : public QWidget
{
    Q_OBJECT
public:
    BidWatcher( QWidget *parent=0, const char *name=0 );
	~BidWatcher();
private slots:
   void CmUser();
   void CmAddItem();
   void CmTimeSync();
   void CmUpdate();
   void CmHelp();
   void CmExit();
   void CmCancelClicked();
   void changeName(char *, char *,char *, bool, bool, bool, bool, bool, int, char *, bool, bool, bool, int);
   void doubleClick(int);
   void addNewItem(char *);
   void timer1Up();
   void timer2Up();
   void timer3Up();
   void timer4Up();
   void CmMenu();
   void CmDetails();
   void CmViewFile();
   void CmViewEnded();
   void UpdateItem();
   void launchBrowser( int choice );
   void deleteItem();
   void flush();
   void setupSnipe(BidMakeup_type &,int);
   void CmBid();
   void CmUnBid();
   void ClearSnipe(int);
   void CmWeb();
protected:
   void resizeEvent( QResizeEvent * );
signals:
private:
	NewListBox      * aucList;
	QLabel          * brightLabel;
	QLabel          * statLabel;
	QLabel          * timeLabel;
        QLabel 		* fontLabel;
	QLabel          * bidLabel;  //dbs
   QLabel          * columnHeaderLabel;
	QToolButton     * cancelButton;
	QToolButton     * UserButton;
	QToolButton     * AddButton;
	QToolButton     * ClockButton;
	QToolButton     * UpdateButton;
	QToolButton     * ExitButton;
	QToolButton     * HelpButton;
	QTimer          * TIMER1;
	QTimer          * TIMER2;
	QTimer          * TIMER3;
	QTimer          * TIMER4;
	ConfigMenu      * firstMenu;
	QGetNumber      * getNumWin;
	QPopupMenu      * listSelect; 
	BidWindow       * bidWindow;
   DetailsWindow   * detailsWindow;
   QFileView       * viewFileWindow;
   QEndedView      * viewEndedWindow;
	
   int             aucIdx;
	volatile bool            updateInProgress;
	bool            firstRun;
	bool            checkForSnipe;
	int             Timer1Idx;
	int             Timer2Idx;
	int             bidIdx;
	int             timeDiff;
   int             snipeDelay; 
	int		fontSize;
   char            authID[76];
   char            authPASS[76];
   char            browserPATH[200];
   char		   emailPATH[200];
   char            ebayPATH[65];
   char            ebaymPATH[65];
   char            currentBid[15];
   char		   buyitnowPrice[15];
   bool            trackListings;
   bool            trackBids;
   bool            autoDelete;
   bool            doStartup;
   bool            soundOn;
	bool		earlycheck;
	bool		boldcheck;
	bool		fixedcheck;
	bool            clockIsSet;
	struct 		    BidMakeup_type BidMakeup[MAXAUCS+1];
	string30        BidList[MAXAUCS + 1];
	QPixmap         brightLed;
	QPixmap         dimLed;
	QPixmap         id_btn;
	QPixmap         add_btn;
	QPixmap         clock_btn;
	QPixmap         update_btn;
	QPixmap         help_btn;
	QPixmap         exit_btn;
	QPixmap         canc_btn;
	QPixmap		redDot;
	QPixmap         blueDot;
	QPixmap         blackDot;
	QBitmap         maskDot;
	QPixmap         greenDot;
	QPixmap         snipeDot;
	QBitmap         maskSnipe;
	QPalette        yellowPalette;
	QPalette        greenPalette;
	QPalette        redPalette;
	QPalette	grayPalette;

	void            TimeSync();
	void            DoSnipe(int);
	void            UpdateListItem(int);
	void	        UpdateList();
   void          ArrangeList();
	void	        DeleteAuction(int);
	void 	        ReadaucFile();
	bool		bigUpdate();
	bool            UpdateAll(bool);
	int		timeToNextEnd();
	void            UpdateLatest();
	int 	        GetUserBids();
	int             GetUserListings();
	int             UpdateAuction(BidMakeup_type &);  
   void          WriteLog(int);      
   void           WriteAucFile();  	
	void		ReadSPFile();
};
void getAdultCookie(char *, char *);
#define TYPE_EBAY 0
#define TYPE_EBAYMOTORSCAR 1  //Ebay motors auctions that are cars
#define TYPE_EBAYMOTORS 0     //Auctions on motors that aren't cars are normal
#endif // AUCTION_H
